#-----------------------------------------------------
# module Audio
# dxrubyのclass Soundを利用して、
# ゲーム内で使用されるBGMや効果音を管理するモジュール。
#-----------------------------------------------------

# dxrubyで扱える音量の最大/最小値
MAX_VOLUME = 255
MIN_VOLUME = 0 # ここを変更すると、色々ずれるから変えないで。
DEFAULT_VOLUME = 230

# ゲーム内で表示される音量の最大/最小値
MAX_DVOLUME = 100
MIN_DVOLUME = 0
DEFAULT_DVOLUME = 100

module Audio
  @master_volume = MAX_DVOLUME # マスターボリュームの初期値
  @bgm_volume = MAX_DVOLUME # BGMボリュームの初期値
  @se_volume = MAX_DVOLUME # SEボリュームの初期値
  @moan_volume = MAX_DVOLUME # 戦闘SEボリュームの初期値
  @bgm_list ={}
  @se_list ={}
  @moan_list ={}
  # ボリュームの設定
  def self.set_init_volume
    @bgm_volume = $system.bgm_volume / 2
    @bgm_volume += 50 if @bgm_volume > 0
    @se_volume = $system.se_volume / 2
    @se_volume += 50 if @se_volume > 0
    @moan_volume = $system.moan_volume / 2
    @moan_volume += 50 if @moan_volume > 0
    if @sound_bgm
      @sound_bgm.set_volume(100, 0) if !@sound_bgm.disposed?
    end
    # フェードアウト処理用
    @fade_duration = 0.0
    @set_vlm = 0.0
    @fade_vlm = 0.0
    @fade_bgm_key = nil
  end
  # @master_volumeのrateを取得
  def self.rate_master_volume
    @master_volume.to_f / (MAX_DVOLUME - MIN_DVOLUME).to_f
  end
  # @bgm_volumeのrateを取得
  def self.rate_bgm_volume
    @bgm_volume.to_f / (MAX_DVOLUME - MIN_DVOLUME).to_f
  end
  # @se_volumeのrateを取得
  def self.rate_se_volume
    @se_volume.to_f / (MAX_DVOLUME - MIN_DVOLUME).to_f
  end
  # @moan_volumeのrateを取得
  def self.rate_moan_volume
    @moan_volume.to_f / (MAX_DVOLUME - MIN_DVOLUME).to_f
  end
  # 引数のrateをdxrubyのボリューム値に適用して返す
  def self.convert_volume_rate(rate)
    rate * MAX_VOLUME
  end
  # 更新（フェードアウト処理用）
  def self.update
    unless @fade_bgm_key.nil?
      @set_vlm = @set_vlm - @fade_vlm
      @bgm_list[@fade_bgm_key].fadeout(@set_vlm)
      if @fade_duration > 1
        @fade_duration -= 1
      else
        self.stop_bgm
        @fade_duration = 0.0
        @set_vlm = 0.0
        @fade_vlm = 0.0
        @fade_bgm_key = nil
      end
    end
  end
  # フェード処理をしていないか？
  def self.not_fade?
    return @fade_bgm_key.nil?
  end
  # BGMリストの参照
  def self.bgm_list
    @bgm_list
  end
  # BGMリストへSoundオブジェクトを追加
  def self.add_bgm_list(key, path, private_vlm = DEFAULT_VOLUME)
    return if !key.is_a?(Symbol) || !path.is_a?(String)
    @sound_bgm = path.empty? ? nil : Sound_Bgm.new(path, private_vlm)
    @bgm_list[key] = @sound_bgm
  end
  # BGMリストから再生
  def self.play_bgm_list(key)
    self.stop_bgm
    self.set_init_volume
    return if @bgm_list[key].nil?
    @bgm_list[key].set_volume(DEFAULT_DVOLUME)
    @bgm_list[key].loop_count=(0) if key == :end
    @bgm_list[key].play
  end
  # BGMをリストに追加しつつ再生
  def self.play_bgm(key, path, private_vlm = DEFAULT_VOLUME)
    self.stop_bgm
    self.add_bgm_list(key, path, private_vlm)
    self.play_bgm_list(key)
  end
  # BGMの停止
  def self.stop_bgm
    @bgm_list.each{|key, bgm| bgm.stop if !bgm.nil?}
  end
  # BGMのフェードアウト
  def self.fadeout_bgm(key, duration)
    return if @bgm_list[key].nil?
    if duration == 0
      @bgm_list[key].fadeout(0)
    else
      @set_vlm = @bgm_list[key].get_volume
      @fade_vlm = @set_vlm.to_f / duration.to_f / 2.0
      @fade_duration = duration
      @fade_bgm_key = key
    end
  end
  # BGMの消去
  def self.delete_bgm(key)
    self.stop_bgm
    return if @bgm_list[key].nil?
    @bgm_list[key].dispose if @bgm_list[key].instance_of?(Sound_Bgm)
    @bgm_list.delete(key)
  end
  # SEリストの参照
  def self.se_list
    @se_list
  end
  # SEリストへの追加
  def self.add_se_list(key, path, private_vlm = DEFAULT_VOLUME, count)
    @sound_se = Sound_SE.new(path, private_vlm, count)
    @se_list[key] = @sound_se
  end
  # SEリストから再生
  def self.play_se_list(key, rate = nil)
    self.stop_se
    @se_volume = $system.se_volume / 2
    @se_volume += 50 if @se_volume > 0
    @se_list[key.to_sym].set_volume(DEFAULT_DVOLUME, 0, false, rate)
    @se_list[key.to_sym].play
  end
  # SEをリストに追加しつつ再生
  def self.play_se(key, path, private_vlm = DEFAULT_VOLUME, loop = false)
    self.stop_se
    self.add_se_list(key, path, private_vlm, loop ? -1 : 1)
    self.play_se_list(key)
  end
  # SEの停止
  def self.stop_se
    @se_list.each{|key, se| se.stop}
  end
  # 戦闘SEリストの参照
  def self.moan_list
    @moan_list
  end
  # 戦闘SEリストへの追加
  def self.add_moan_list(key, path, private_vlm = DEFAULT_VOLUME, count)
    @sound_moan = Sound_Moan.new(path, private_vlm, count)
    @moan_list[key] = @sound_moan
  end
  # 戦闘SEリストから再生
  def self.play_moan_list(key, private_vlm)
    self.stop_moan
    @moan_volume = ($system.moan_volume * (private_vlm / 100.0)) / 2
    @moan_volume += 50 if @moan_volume > 0
    @moan_list[key.to_sym].set_volume(DEFAULT_DVOLUME, 0, false, nil)
    @moan_list[key.to_sym].play
  end
  # 戦闘SEをリストに追加しつつ再生
  def self.play_moan(key, path, private_vlm = DEFAULT_VOLUME, loop = false)
    self.add_moan_list(key, path, private_vlm, loop ? -1 : 1)
    self.play_moan_list(key, private_vlm)
  end
  # 戦闘SEの停止
  def self.stop_moan
    return if @moan_list.nil?
    @moan_list.each{|key, se| se.stop}
  end
  # 直接指定
  def self.play_input(se_id)
    self.play_se_list(se_id)
  end
  # カーソル
  def self.play_cursor
    self.play_se_list(:select)
  end
  # 決定
  def self.play_ok
    self.play_se_list(:ok)
  end
  # キャンセル
  def self.play_cancel
    self.play_se_list(:cancel)
  end
  # ブザー
  def self.play_buzzer
    self.play_se_list(:buzzer)
  end
  # 装備
  def self.play_plugin
    self.play_se_list(:set1)
  end
  # 絞込み
  def self.play_narrow
    self.play_se_list(:select1)
  end
  # ダメージ
  def self.play_damage
    damege_se = [:damege1, :damege2]
    self.play_se_list(damege_se[rand(2)])
  end
  # 陵辱モード
  def self.play_bind
    self.play_se_list(:bind1)
  end
  # コマンド入力
  def self.play_release
    self.play_se_list(:command1)
  end
  # コマンド入力成功
  def self.play_success_release
    self.play_se_list(:success)
  end
  # コマンド入力失敗
  def self.play_miss_release
    self.play_se_list(:miss)
  end
  # 転送
  def self.play_trancepose
    self.play_se_list(:sting1)
  end
  # 射精
  def self.play_sperm
    self.play_se_list(:sperm1)
  end
  # 敗北回想BGM
  def self.play_recollect(id)
    if self.recollect_special_bgm_enemy.include?(id)
      self.play_bgm(:enemy, self.play_enemy_bgm(id))
    else
      self.play_bgm(:enemy, './data/snd/bgm/快楽に溺れて.wav')
    end
  end
  # 専用敵BGM
  def self.play_enemy_bgm(id)
    case id
    when 1 ; return './data/snd/bgm/未来への抵抗.wav'
    when 40,49,89,90,91,92,93,97,98,99,139 ; return './data/snd/bgm/生と死のテクノロジー.wav'
    when 44,172 ; return './data/snd/bgm/それでも.wav'
    when 51,52,110,111 ; return './data/snd/bgm/Say,love me.wav'
    when 66
      return './data/snd/bgm/宵闇の雪の中で.wav' unless $game_switches[7504]
    when 71 ; return './data/snd/bgm/あなたの背中に.wav'
    when 78 ; return './data/snd/bgm/世界を描いて.wav'
    when 79,80,81,123,125 ; return './data/snd/bgm/未来への挑戦.wav'
    when 94
      return './data/snd/bgm/ANIMALOID.wav' unless $game_switches[1692]
    when 95,158 ; return './data/snd/bgm/堕ちた想いに.wav'
    when 145,157 ; return './data/snd/bgm/堕ちた想いに.wav' if BattleManager.on_simulation?
    when 100 ; return './data/snd/bgm/deep blue-feat.ANIMALOID.wav' unless $game_switches[8352]
    when 103,104,121,137,141,142,146,157,161,166,169 ; return './data/snd/bgm/堕ちてゆく快楽のさなかに.wav'
    when 153 ; return './data/snd/bgm/堕ちてゆく快楽のさなかに.wav' if BattleManager.on_simulation?
    when 105 ; return './data/snd/bgm/ユーロ・ラグニア-EscMode.wav' if BattleManager.on_simulation?
    when 106 ; return './data/snd/bgm/ユーロ・ラグニア-EscMode.wav'
    when 143,148,171,176 ; return './data/snd/bgm/distorted Gear.wav'
    when 109 ; return './data/snd/bgm/distorted Gear.wav' if BattleManager.on_simulation?
    when 115,119,120 ; return './data/snd/bgm/忍び生きる者たち.wav' unless $game_switches[1888]
    when 138 ; return './data/snd/bgm/生と死のテクノロジー.wav' unless $game_switches[1967]
    when 140 ; return './data/snd/bgm/Vanishing.wav' unless $game_switches[1978]
    when 142 ; return './data/snd/bgm/deep blue-feat.ANIMALOID.wav' if BattleManager.on_simulation?
    when 147 ; return './data/snd/bgm/泣く事も忘れて.wav' unless $game_switches[1992]
    when 142 ; return './data/snd/bgm/UnNaturaloid-feat.ANIMALOID.wav'
    when 156 ; return './data/snd/bgm/科学と欲望渦巻く街.wav'
    when 162,167 ; return './data/snd/bgm/UnNaturaloid-feat.ANIMALOID.wav' if BattleManager.on_simulation?
    when 168 ; return './data/snd/bgm/sarah=el-xion.wav' if BattleManager.on_simulation?
    when 169 ; return './data/snd/bgm/殺戮狂詩曲.wav' unless $game_switches[2065]
    when 173 ; return './data/snd/bgm/今は遠い故郷へ.wav'
    when 174 ; return './data/snd/bgm/究極の名のもとに.wav' if BattleManager.on_simulation?
    when 175 ; return './data/snd/bgm/rit al fine.wav' if BattleManager.on_simulation?
    else ; return nil
    end
  end
  # 敗北回想で特殊BGMを流す相手のID
  def self.recollect_special_bgm_enemy
    return [44,71,78,95,103,104,121,137,141,142,143,146,148,157,158,161,166,169,171,172,173,176]
  end
end
